from __future__ import absolute_import

from tests.tests import TestMYQL
from tests.tests import TestWeather
from tests.tests import TestStockScraper
from tests.tests import TestTable
from tests.tests import TestOAuth
from tests.tests import TestSocial
from tests.tests import TestFilters
from tests.tests import TestRemoteFilters
from tests.tests import TestFuncFilters
from tests.tests import TestPaging
from tests.tests import TestMultiQuery
